'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const ts = require("typescript");
const tslint_1 = require("tslint");
const tsLinter_1 = require("../tsLinter");
const contentRenderer_1 = require("../contentRenderer");
const utils_1 = require("../utils");
const configFileNormalizer_1 = require("../configFileNormalizer");
const mock = require('mock-fs');
const assert = require('power-assert');
describe('TsLinter', () => {
    const linterPath = './';
    const targetPath = '/base/path/';
    const templateFile = path.join(linterPath, contentRenderer_1.ContentRenderer.templateFileName);
    class MockedConfigFileNormalizer extends configFileNormalizer_1.ConfigFileNormalizer {
        normalize(inPath, altRulesDirectory) { return super.baseOutDir; }
    }
    describe('.originalConfigPath', () => {
        const rules = [];
        it('returns the specified file on the base path', done => {
            // Given
            const config = 'configured.json';
            const codeClimateConfig = { config, include_paths: [] };
            const expected = path.join(targetPath, config);
            mock({ [expected]: '{}', [templateFile]: 'exists' });
            // When
            const tsLinter = new tsLinter_1.TsLinter({ targetPath, linterPath, codeClimateConfig, rules }, new MockedConfigFileNormalizer(targetPath));
            // Then
            assert.equal(tsLinter.originalConfigPath, expected);
            mock.restore();
            done();
        });
        it('returns path with default file name on the base path', done => {
            // Given
            const codeClimateConfig = { include_paths: [] };
            const expected = path.join(targetPath, tsLinter_1.TsLinter.defaultTsLintFileName);
            mock({ [expected]: '{}', [templateFile]: 'exists' });
            // When
            const tsLinter = new tsLinter_1.TsLinter({ targetPath, linterPath, codeClimateConfig, rules }, new MockedConfigFileNormalizer(targetPath));
            // Then
            assert.equal(tsLinter.originalConfigPath, expected);
            mock.restore();
            done();
        });
        it('returns default path if the specified file does not exist on the base path', done => {
            // Given
            const codeClimateConfig = { include_paths: [] };
            const expected = path.join(linterPath, tsLinter_1.TsLinter.defaultTsLintFileName);
            mock({ [expected]: '{}', [templateFile]: 'exists' });
            // When
            const tsLinter = new tsLinter_1.TsLinter({ targetPath, linterPath, codeClimateConfig, rules }, new MockedConfigFileNormalizer(targetPath));
            // Then
            assert.equal(tsLinter.originalConfigPath, expected);
            mock.restore();
            done();
        });
        it('returns default path if default tslint.json does not exist on the base path', done => {
            // Given
            const codeClimateConfig = { include_paths: [] };
            const expected = path.join(linterPath, tsLinter_1.TsLinter.defaultTsLintFileName);
            mock({ [expected]: '{}', [templateFile]: 'exists' });
            // When
            const tsLinter = new tsLinter_1.TsLinter({ targetPath, linterPath, codeClimateConfig, rules }, new MockedConfigFileNormalizer(targetPath));
            // Then
            assert.equal(tsLinter.originalConfigPath, expected);
            mock.restore();
            done();
        });
    });
    describe('.listFiles()', () => {
        const rules = [];
        it('returns file list', done => {
            // Given
            const tsLintPath = path.join(targetPath, tsLinter_1.TsLinter.defaultTsLintFileName);
            mock({ [tsLintPath]: '{}', [templateFile]: 'exists' });
            const codeClimateConfig = { include_paths: [] };
            const tsLinter = new tsLinter_1.TsLinter({ targetPath, linterPath, codeClimateConfig, rules }, new MockedConfigFileNormalizer(targetPath));
            tsLinter.fileMatcher = new class {
                matchFiles() {
                    return ['file.ts'];
                }
            };
            // When
            const actual = tsLinter.listFiles();
            // Then
            assert.deepStrictEqual(actual, ['file.ts']);
            mock.restore();
            done();
        });
    });
    describe('.lint()', () => {
        // Mocked version of Linter
        class MockedLinter extends tslint_1.Linter {
            constructor(options, dummyFailures) {
                super(options);
                this.dummyFailures = dummyFailures;
            }
            lint(fileName, source, configuration) { }
            getResult() {
                return {
                    errorCount: 2,
                    warningCount: 0,
                    failures: this.dummyFailures,
                    format: 'json',
                    output: 'output'
                };
            }
        }
        // Mocked version of TsLinter
        class MockedTsLinter extends tsLinter_1.TsLinter {
            constructor(codeClimateConfig, dummyFailures) {
                super({ targetPath, linterPath, codeClimateConfig, rules }, new MockedConfigFileNormalizer(linterPath));
                this.dummyFailures = dummyFailures;
            }
            listFiles() { return ['file.ts']; }
            createLinter() { return new MockedLinter(this.linterOption, this.dummyFailures); }
        }
        function createSourceFile(fileName) {
            const source = `'use strict';
      var unused = 32;
      let object = {
        c: 42,
        a: false
      };
      `;
            return ts.createSourceFile(`${targetPath}${fileName}`, source, ts.ScriptTarget.ES2016);
        }
        function mockPaths() {
            const tslintPath = path.join(linterPath, tsLinter_1.TsLinter.defaultTsLintFileName);
            mock({
                [tslintPath]: '{}',
                'file.ts': '',
                [templateFile]: 'exists'
            });
        }
        function createIssue() {
            return {
                type: 'issue',
                check_name: ruleName,
                content: {
                    body: '' // omit testing
                },
                description: 'some failure',
                categories: ['Style'],
                remediation_points: 50000,
                location: {
                    path: fileName,
                    positions: {
                        begin: { line: 1, column: 1 },
                        end: { line: 1, column: 1 }
                    }
                }
            };
        }
        function assertLintResult(actual, expected, done) {
            actual
                .map((result) => {
                result.content = { body: '' };
                return result;
            })
                .toArray()
                .subscribe(actual => {
                // Then
                assert.deepStrictEqual(actual, expected);
                done();
            }, assert.fail.bind(assert), mock.restore.bind(mock));
        }
        const ruleName = 'foo-rule';
        const rules = [
            {
                description: 'foo',
                ruleName,
                type: 'style',
                typescriptOnly: true,
                options: null,
                optionsDescription: '',
                optionExamples: []
            }
        ];
        const fileName = 'failed.ts';
        const file = createSourceFile(fileName);
        it('passes', done => {
            // Given
            mockPaths();
            const location = {
                path: fileName,
                positions: {
                    begin: { line: 1, column: 2 },
                    end: { line: 2, column: 7 }
                }
            };
            const issue = Object.assign({}, createIssue(), { location });
            const failure = new tslint_1.RuleFailure(file, 1, 20, issue.description, ruleName);
            const tsLinter = new MockedTsLinter({ include_paths: [] }, [failure]);
            const expected = [issue];
            // When
            const actual = tsLinter.lint();
            // Then
            assertLintResult(actual, expected, done);
        });
        it('doesn\'t report warnings if ignore_warnings option is given', done => {
            // Given
            mockPaths();
            const warningIssue = Object.assign({}, createIssue(), { description: 'some warning' });
            const errorIssue = Object.assign({}, createIssue(), { description: 'some error' });
            const warning = new tslint_1.RuleFailure(file, 0, 0, warningIssue.description, ruleName);
            warning.setRuleSeverity('warning');
            const error = new tslint_1.RuleFailure(file, 0, 0, errorIssue.description, ruleName);
            error.setRuleSeverity('error');
            const tsLinter = new MockedTsLinter({ include_paths: [], ignore_warnings: true }, [warning, error]);
            const expected = [errorIssue]; // warningIssue is not included
            // When
            const actual = tsLinter.lint();
            // Then
            assertLintResult(actual, expected, done);
        });
        it('returns runtime-error issue when failing to convert a rule failure to an issue', done => {
            // Given
            mockPaths();
            const failure = new tslint_1.RuleFailure(file, 0, 0, 'whatever', 'non-existent');
            const tsLinter = new MockedTsLinter({ include_paths: [] }, [failure]);
            const error = new Error();
            tsLinter.issueConverter.convert = function failingConvert(failure) {
                throw error;
            };
            const issue = Object.assign({}, utils_1.default.createIssueFromError(error, tsLinter.getRelativeFilePath('file.ts')), { content: { body: '' } });
            const expected = [issue];
            // When
            const actual = tsLinter.lint();
            // Then
            assertLintResult(actual, expected, done);
        });
    });
    describe('.getRelativeFilePath', () => {
        // Mocked version of TsLinter
        class MockedTsLinter extends tsLinter_1.TsLinter {
            constructor(targetPath) {
                super({ targetPath, linterPath, codeClimateConfig: { include_paths: [] }, rules: [] }, new MockedConfigFileNormalizer(linterPath));
            }
        }
        it('returns correct path', () => {
            const tsLinter = new MockedTsLinter('/code/src');
            assert(tsLinter.getRelativeFilePath('/code/src/foo.ts') === 'foo.ts');
            assert(tsLinter.getRelativeFilePath('/code/src/foo/bar.ts') === 'foo/bar.ts');
            assert(tsLinter.getRelativeFilePath('/code/tmp/foo.ts') === '../tmp/foo.ts');
            assert(tsLinter.getRelativeFilePath('/tmp/foo.ts') === '../../tmp/foo.ts');
        });
    });
});
//# sourceMappingURL=tsLinter.spec.js.map